@@ -34,11 +34,11 @@
 #include <gdk/gdkkeysyms.h>
 #include <gio/gdesktopappinfo.h>
 
+
+
 /* for DBG/TRACE */
 #include <libxfce4util/libxfce4util.h>
 
-
-
 struct _MprisMenuItemPrivate {
   GtkWidget *title_label;
   GtkWidget *artist_label;
@@ -52,6 +52,7 @@ struct _MprisMenuItemPrivate {
   gboolean   can_pause;
   gboolean   can_go_next;
   gboolean   can_raise;
+  gboolean   can_raise_wnck;
 
   gboolean   is_running;
   gboolean   is_playing;
@@ -89,6 +90,11 @@ G_GNUC_END_IGNORE_DEPRECATIONS
 /* Static Declarations */
 static void         mpris_menu_item_finalize                (GObject        *object);
 static void         mpris_menu_item_raise                   (MprisMenuItem  *item);
+
+#ifdef HAVE_WNCK
+static void         mpris_menu_item_raise_window            (MprisMenuItem  *item);
+#endif
+
 static void         mpris_menu_item_launch                  (MprisMenuItem  *item);
 static void         mpris_menu_item_raise_or_launch         (MprisMenuItem  *item);
 static GtkWidget *  mpris_menu_item_get_widget_at_event     (MprisMenuItem  *item,
@@ -162,7 +168,6 @@ mpris_menu_item_new_from_player_name (const gchar *player)
   filename = find_desktop_entry (player);
   if (filename == NULL)
     {
-      g_free (filename);
       return NULL;
     }
 
@@ -339,6 +344,21 @@ mpris_menu_item_set_can_raise (MprisMenuItem *item,
 
 
 
+void
+mpris_menu_item_set_can_raise_wnck (MprisMenuItem *item,
+                                    gboolean       can_raise)
+{
+  MprisMenuItemPrivate *priv;
+
+  g_return_if_fail (IS_MPRIS_MENU_ITEM (item));
+
+  priv = GET_PRIVATE (item);
+
+  priv->can_raise_wnck = can_raise;
+}
+
+
+
 void
 mpris_menu_item_set_is_running (MprisMenuItem *item,
                                 gboolean       running)
@@ -449,7 +469,7 @@ mpris_menu_item_class_init (MprisMenuItemClass *item_class)
    * @menuitem: the #MprisMenuItem for which the value changed
    * @value: the mpris signal to emit
    *
-   * Emitted whenever the a media button is clicked.
+   * Emitted whenever a media button is clicked.
    */
   signals[MEDIA_NOTIFY] = g_signal_new ("media-notify",
                                         TYPE_MPRIS_MENU_ITEM,
@@ -527,8 +547,19 @@ mpris_menu_item_raise (MprisMenuItem *item)
 
   priv = GET_PRIVATE (item);
 
-  if (priv->is_running && priv->can_raise)
-    media_notify (item, "Raise");
+  if (priv->is_running)
+    {
+      if (priv->can_raise)
+        {
+          media_notify (item, "Raise");
+        }
+#ifdef HAVE_WNCK
+      else if (priv->can_raise_wnck)
+        {
+          media_notify (item, "RaiseWnck");
+        }
+#endif
+    }
 }
 
 
