@@ -26,6 +26,11 @@
 #include <glib.h>
 #include <gtk/gtk.h>
 
+#ifdef HAVE_WNCK
+#define WNCK_I_KNOW_THIS_IS_UNSTABLE = 1
+#include <libwnck/libwnck.h>
+#endif
+
 #include "pulseaudio-mpris-player.h"
 
 
@@ -37,6 +42,7 @@ struct _PulseaudioMprisPlayer
   GDBusConnection  *dbus_connection;
   GDBusProxy       *dbus_props_proxy;
   GDBusProxy       *dbus_player_proxy;
+  GDBusProxy       *dbus_playlists_proxy;
   gchar            *dbus_name;
 
   gchar            *player;
@@ -57,6 +63,10 @@ struct _PulseaudioMprisPlayer
   PlaybackStatus    playback_status;
 
   guint             watch_id;
+
+  GHashTable       *playlists;
+
+  gulong            xid;
 };
 
 struct _PulseaudioMprisPlayerClass
@@ -72,10 +82,11 @@ struct _PulseaudioMprisPlayerClass
 static void pulseaudio_mpris_player_finalize     (GObject               *object);
 static void pulseaudio_mpris_player_dbus_connect (PulseaudioMprisPlayer *player);
 
+static GVariant *pulseaudio_mpris_player_playlists_get_playlists  (PulseaudioMprisPlayer *player);
+static void      pulseaudio_mpris_player_parse_playlists          (PulseaudioMprisPlayer *player,
+                                                                   GVariant              *playlists);
 
-
-enum
-{
+enum {
   CONNECTION,
   PLAYBACK_STATUS,
   METADATA,
@@ -143,6 +154,8 @@ pulseaudio_mpris_player_parse_metadata (PulseaudioMprisPlayer *player,
     {
       if (0 == g_ascii_strcasecmp (key, "xesam:title"))
         {
+          if (player->title != NULL)
+            g_free (player->title);
           player->title = g_strdup(g_variant_get_string(value, NULL));
         }
       else if (0 == g_ascii_strcasecmp (key, "xesam:artist"))
@@ -150,6 +163,9 @@ pulseaudio_mpris_player_parse_metadata (PulseaudioMprisPlayer *player,
           artists = g_variant_dup_strv (value, NULL);
           if (artists != NULL)
             {
+              if (player->artist != NULL)
+                g_free(player->artist);
+
               if (g_strv_length (artists) > 0)
                 {
                   player->artist = g_strdup (artists[0]);
@@ -158,6 +174,7 @@ pulseaudio_mpris_player_parse_metadata (PulseaudioMprisPlayer *player,
                 {
                   player->artist = g_strdup ("");
                 }
+
               g_strfreev (artists);
             }
         }
@@ -166,6 +183,59 @@ pulseaudio_mpris_player_parse_metadata (PulseaudioMprisPlayer *player,
 
 
 
+#ifdef HAVE_WNCK
+static void
+pulseaudio_mpris_player_get_xid (PulseaudioMprisPlayer *player)
+{
+  WnckScreen           *screen = NULL;
+  GList                *window = NULL;
+
+  if (player->xid > 0L)
+    return;
+
+  screen = wnck_screen_get_default ();
+  if (screen != NULL)
+    {
+      wnck_screen_force_update (screen);
+      if (player->xid == 0L)
+        {
+          for (window = wnck_screen_get_windows (screen); window != NULL; window = window->next)
+            {
+              if (0 == g_strcmp0 (player->player_label, wnck_window_get_name (WNCK_WINDOW (window->data))))
+                {
+                  player->xid = wnck_window_get_xid (WNCK_WINDOW (window->data));
+                  if (player->xid > 0L)
+                    return;
+                }
+            }
+        }
+    }
+}
+
+
+
+/**
+ * Alternative "Raise" method.
+ * Some media players (e.g. Spotify) do not support the "Raise" method.
+ * This workaround utilizes libwnck to find the correct window and raise it.
+ */
+static void
+pulseaudio_mpris_player_raise_wnck (PulseaudioMprisPlayer *player)
+{
+  WnckWindow           *window = NULL;
+
+  pulseaudio_mpris_player_get_xid (player);
+
+  if (player->xid == 0L)
+    return;
+
+  window = wnck_window_get (player->xid);
+  wnck_window_activate (window, 0);
+}
+#endif
+
+
+
 void
 pulseaudio_mpris_player_call_player_method (PulseaudioMprisPlayer *player,
                                             const gchar           *method)
@@ -176,6 +246,10 @@ pulseaudio_mpris_player_call_player_method (PulseaudioMprisPlayer *player,
 
   if (g_strcmp0 (method, "Raise") == 0)
     iface = "org.mpris.MediaPlayer2";
+#ifdef HAVE_WNCK
+  else if (g_strcmp0 (method, "RaiseWnck") == 0)
+    return pulseaudio_mpris_player_raise_wnck (player);
+#endif
   else if (g_strcmp0 (method, "Quit") == 0)
     iface = "org.mpris.MediaPlayer2";
   else
@@ -210,8 +284,40 @@ pulseaudio_mpris_player_call_player_method (PulseaudioMprisPlayer *player,
 
 
 
+GList *
+pulseaudio_mpris_player_get_playlists (PulseaudioMprisPlayer *player)
+{
+  return g_hash_table_get_keys (player->playlists);
+}
+
+
+
+void pulseaudio_mpris_player_activate_playlist (PulseaudioMprisPlayer *player,
+                                                const gchar           *playlist)
+{
+  gchar *path = g_hash_table_lookup (player->playlists, playlist);
+
+  if (path != NULL)
+    {
+      g_dbus_connection_call (player->dbus_connection,
+                              player->dbus_name,
+                              "/org/mpris/MediaPlayer2",
+                              "org.mpris.MediaPlayer2.Playlists",
+                              "ActivatePlaylist",
+                              g_variant_new("(o)", path),
+                              NULL,
+                              G_DBUS_CALL_FLAGS_NONE,
+                              -1,
+                              NULL,
+                              NULL,
+                              NULL);
+    }
+}
+
+
+
 static GVariant *
-pulseaudio_mpris_player_get_all_player_properties (PulseaudioMprisPlayer *player)
+pulseaudio_mpris_player_get_all_player_properties(PulseaudioMprisPlayer *player)
 {
   GVariantIter iter;
   GVariant *result, *child = NULL;
@@ -268,6 +374,35 @@ pulseaudio_mpris_player_get_all_media_player_properties (PulseaudioMprisPlayer *
 
 
 
+static GVariant *
+pulseaudio_mpris_player_playlists_get_playlists (PulseaudioMprisPlayer *player)
+{
+  GVariantIter iter;
+  GVariant *result, *child = NULL;
+
+  result = g_dbus_connection_call_sync(player->dbus_connection,
+                                       player->dbus_name,
+                                       "/org/mpris/MediaPlayer2",
+                                       "org.mpris.MediaPlayer2.Playlists",
+                                       "GetPlaylists",
+                                       g_variant_new("(uusb)", (guint32)0, (guint32)5, "Played", TRUE),
+                                       G_VARIANT_TYPE("(a(oss))"),
+                                       G_DBUS_CALL_FLAGS_NONE,
+                                       -1,
+                                       NULL,
+                                       NULL);
+
+  if (result)
+  {
+    g_variant_iter_init(&iter, result);
+    child = g_variant_iter_next_value(&iter);
+  }
+
+  return child;
+}
+
+
+
 static void
 pulseaudio_mpris_player_parse_playback_status (PulseaudioMprisPlayer *player,
                                                const gchar           *playback_status)
@@ -320,6 +455,16 @@ pulseaudio_mpris_player_parse_player_properties (PulseaudioMprisPlayer *player,
           pulseaudio_mpris_player_parse_metadata (player, value);
           g_signal_emit (player, signals[METADATA], 0, NULL);
         }
+      else if (0 == g_ascii_strcasecmp(key, "ActivePlaylist") || 0 == g_ascii_strcasecmp(key, "PlaylistCount"))
+        {
+          /* Playlists */
+          GVariant *reply = pulseaudio_mpris_player_playlists_get_playlists(player);
+          if (reply)
+          {
+            pulseaudio_mpris_player_parse_playlists(player, reply);
+            g_variant_unref(reply);
+          }
+        }
     }
 
   if (playback_status != NULL)
@@ -352,6 +497,28 @@ pulseaudio_mpris_player_parse_media_player_properties (PulseaudioMprisPlayer *pl
 
 
 
+static void
+pulseaudio_mpris_player_parse_playlists (PulseaudioMprisPlayer *player,
+                                         GVariant *playlists)
+{
+  GVariantIter iter;
+
+  gchar *path;
+  const gchar *name;
+  const gchar *icon;
+
+  g_hash_table_remove_all (player->playlists);
+
+  g_variant_iter_init(&iter, playlists);
+
+  while (g_variant_iter_loop(&iter, "(oss)", &path, &name, &icon))
+  {
+    g_hash_table_insert (player->playlists, g_strdup (name), g_strdup (path));
+  }
+}
+
+
+
 static void
 pulseaudio_mpris_player_on_dbus_property_signal (GDBusProxy *proxy,
                                                  gchar      *sender_name,
@@ -370,11 +537,17 @@ pulseaudio_mpris_player_on_dbus_property_signal (GDBusProxy *proxy,
   g_variant_iter_init (&iter, parameters);
 
   child = g_variant_iter_next_value (&iter); /* Interface name. */
-  g_variant_unref (child);
+  if (child)
+    {
+      g_variant_unref (child);
+    }
 
   child = g_variant_iter_next_value (&iter); /* Property name. */
-  pulseaudio_mpris_player_parse_player_properties (player, child);
-  g_variant_unref (child);
+  if (child)
+    {
+      pulseaudio_mpris_player_parse_player_properties (player, child);
+      g_variant_unref(child);
+    }
 }
 
 
@@ -396,12 +569,33 @@ pulseaudio_mpris_player_on_dbus_connected (GDBusConnection *connection,
 
   /* And informs the current status of the player */
   reply = pulseaudio_mpris_player_get_all_player_properties (player);
-  pulseaudio_mpris_player_parse_player_properties (player, reply);
-  g_variant_unref (reply);
+  if (reply)
+    {
+      pulseaudio_mpris_player_parse_player_properties (player, reply);
+      g_variant_unref (reply);
+    }
 
+
+  /* Media player properties */
   reply = pulseaudio_mpris_player_get_all_media_player_properties (player);
-  pulseaudio_mpris_player_parse_media_player_properties (player, reply);
-  g_variant_unref (reply);
+  if (reply)
+    {
+      pulseaudio_mpris_player_parse_media_player_properties (player, reply);
+      g_variant_unref (reply);
+    }
+
+
+  /* Playlists */
+  reply = pulseaudio_mpris_player_playlists_get_playlists (player);
+  if (reply)
+  {
+    pulseaudio_mpris_player_parse_playlists (player, reply);
+    g_variant_unref (reply);
+  }
+
+#ifdef HAVE_WNCK
+  pulseaudio_mpris_player_get_xid (player);
+#endif
 }
 
 
@@ -429,6 +623,7 @@ pulseaudio_mpris_player_on_dbus_lost (GDBusConnection *connection,
 
   player->title           = NULL;
   player->artist          = NULL;
+  player->xid             = 0L;
 
   g_signal_emit (player, signals[CONNECTION], 0, player->connected);
 }
@@ -482,19 +677,22 @@ static void
 pulseaudio_mpris_player_set_details_from_desktop (PulseaudioMprisPlayer *player,
                                                   const gchar           *player_name)
 {
-  GKeyFile  *key_file;
-  gchar     *file;
-  gchar     *full_path;
-  gchar     *filename;
+  GKeyFile  *key_file = NULL;
+  gchar     *file = NULL;
+  gchar     *full_path = NULL;
+  gchar     *filename = NULL;
 
   filename = find_desktop_entry (player_name);
 
+  if (player->player_label != NULL)
+    g_free (player->player_label);
+  if (player->icon_name != NULL)
+    g_free (player->icon_name);
+
   if (filename == NULL)
     {
-      player->player_label = g_strdup(player->player);
-      player->icon_name = "applications-multimedia";
-
-      g_free (filename);
+      player->player_label = g_strdup (player->player);
+      player->icon_name = g_strdup ("applications-multimedia");
       return;
     }
 
@@ -507,18 +705,21 @@ pulseaudio_mpris_player_set_details_from_desktop (PulseaudioMprisPlayer *player,
       gchar *name = g_key_file_get_string (key_file, "Desktop Entry", "Name", NULL);
       gchar *icon_name = g_key_file_get_string (key_file, "Desktop Entry", "Icon", NULL);
 
-      player->player_label = g_strdup(name);
-      player->icon_name = g_strdup(icon_name);
+      player->player_label = g_strdup (name);
+      player->icon_name = g_strdup (icon_name);
 
       g_free (name);
       g_free (icon_name);
     }
   else
     {
-      player->player_label = g_strdup(player->player);
-      player->icon_name = "applications-multimedia";
+      player->player_label = g_strdup (player->player);
+      player->icon_name = g_strdup ("applications-multimedia");
     }
 
+  if (full_path != NULL)
+    g_free (full_path);
+
   g_key_file_free (key_file);
   g_free (file);
 }
@@ -545,6 +746,11 @@ pulseaudio_mpris_player_set_player (PulseaudioMprisPlayer *player,
       g_object_unref (player->dbus_player_proxy);
       player->dbus_player_proxy = NULL;
     }
+  if (player->dbus_playlists_proxy != NULL)
+    {
+      g_object_unref (player->dbus_playlists_proxy);
+      player->dbus_playlists_proxy = NULL;
+    }
 
   /* Clean player */
   if (player->player != NULL)
@@ -626,6 +832,27 @@ pulseaudio_mpris_player_dbus_connect (PulseaudioMprisPlayer *player)
       player->dbus_player_proxy = proxy;
     }
 
+  /* interface=org.mpris.MediaPlayer2.Player */
+  proxy = g_dbus_proxy_new_sync(player->dbus_connection,
+                                G_DBUS_PROXY_FLAGS_NONE,
+                                NULL,
+                                player->dbus_name,
+                                "/org/mpris/MediaPlayer2",
+                                "org.mpris.MediaPlayer2.Playlists",
+                                NULL, /* GCancellable */
+                                &gerror);
+
+  if (proxy == NULL)
+    {
+      g_printerr("Error creating proxy: %s\n", gerror->message);
+      g_error_free(gerror);
+      gerror = NULL;
+    }
+  else
+    {
+      player->dbus_playlists_proxy = proxy;
+    }
+
   player->watch_id = watch_id;
 }
 
@@ -751,6 +978,7 @@ pulseaudio_mpris_player_init (PulseaudioMprisPlayer *player)
   player->dbus_name         = NULL;
   player->dbus_props_proxy  = NULL;
   player->dbus_player_proxy = NULL;
+  player->dbus_playlists_proxy = NULL;
   player->connected         = FALSE;
 
   player->title             = NULL;
@@ -780,6 +1008,7 @@ pulseaudio_mpris_player_finalize (GObject *object)
   player->dbus_name         = NULL;
   player->dbus_props_proxy  = NULL;
   player->dbus_player_proxy = NULL;
+  player->dbus_playlists_proxy = NULL;
   player->connected         = FALSE;
 
   player->title             = NULL;
@@ -795,7 +1024,10 @@ pulseaudio_mpris_player_finalize (GObject *object)
 
   player->watch_id          = 0;
 
-  (*G_OBJECT_CLASS (pulseaudio_mpris_player_parent_class)->finalize) (object);
+  if (player->playlists != NULL)
+    g_hash_table_destroy (player->playlists);
+
+  (*G_OBJECT_CLASS(pulseaudio_mpris_player_parent_class)->finalize)(object);
 }
 
 
@@ -822,5 +1054,7 @@ pulseaudio_mpris_player_new (gchar *name)
   pulseaudio_mpris_player_dbus_connect (player);
   pulseaudio_mpris_player_set_player (player, name);
 
+  player->playlists = g_hash_table_new_full (g_str_hash, g_str_equal, (GDestroyNotify)g_free, (GDestroyNotify)g_free);
+
   return player;
 }
